The problem asks us to find all substrings that have equal number of consecutive count of 0s and 1s. With the given constraints, finding and checking for all substrings would obviously not work and we need to figure out a way to solve it in linear time.

---

✔️ _**Solution - I (Maintain Count of current & previous consecutive characters & Add minimum)**_

The problem can be solved by observing the examples carefully -

```
1. 0011
In this string, consecutive count of binary characters are [2, 2]. We can form a total of 2 substrings.

2. 00011
In this string, consecutive count of binary characters are [3, 2]. Still, we can only form 2 substrings.

3. 000111
Here, consecutive count of binary characters are as - [3,3]. Now, we can form 3 substrings.

4. 00011100
Consecutive count of binary characters are - [3,3,2]. We can form 3 substrings with the first 2 groups of zeros and ones. 
Then we can form 2 substrings with the latter 2 groups. So, a total of 5 substrings.

5. 00011100111
Consecutive count - [3,3,2,3]. Substrings formed - 3 + 2 + 2 = 7
```

We can observe from the above examples that our final count will only depend on the consecutive counts of binary characters. **With each two groups of consecutive characters, the number of substrings that can be formed will be minimum of count among the two groups**.

Now, although we could maintain all the groupings and their counts and then count the number of substrings, we actually don't even need to maintain the consecutive counts in all of the string. We can **just store the current consecutive count and previous consecutive count** and count the substrings on the fly.

1. **`s[i] == s[i - 1]`** : When current character is equal to previous - just increment the current consecutive count.
2. **`s[i] != s[i - 1]`** :Whenever current character is not equal to previous - We know that we atleast have group of 2 different characters having `consecutiveCount >= 1`. The number of substrings that can be formed from these would be equal to minimum of _`currentConsecutiveCount`_ & _`prevConsecutiveCount`_.  
    So just add that amount to `ans`. Now _`prevConsecutiveCount`_ will become equal to _`currentConsecutiveCount`_ and reset the _`currentConsecutiveCount`_ to 1.



---


```python

def countBinarySubstrings(self, s: str) -> int:
	ans, prev, cur = 0, 0, 1
	for i in range(1, len(s)):
		if s[i] != s[i - 1]:
			ans += min(prev, cur)
			prev = cur
			cur = 1
		else:
			cur += 1
	ans += min(prev, cur)
	return ans
	
```

_**Time Complexity :**_ **`O(N)`**, where _`N`_ is the length of given string.  
_**Space Complexity :**_ **`O(1)`**, since only constant space is being used.
